import pygame, random
from pygame.locals import *

from heapq import heappush, heappop

def image(path):
	return pygame.image.load('../images/%s'%path)

north=0
east=1
south=2
west=3

dirs=['n','e','s','w']

size=32

def direction(x,y,x1,y1):
	xd=x-x1
	yd=y-y1
	if abs(xd)>abs(yd):
		if xd>0:
			return west
		else:
			return east
	else:
		if yd>0:
			return north
		return south

class Enemy:
	def xy(self):
		return self.fromsquare.x,self.fromsquare.y
	def directionto(self, x,y):
		return direction(self.fromsquare.x,self.fromsquare.y,x,y)
	def squarefromdirection(self,direction):
		x,y=0,0
		if direction==north:
			x,y=self.fromsquare.x,self.fromsquare.y-1
		if direction==south:
			x,y=self.fromsquare.x,self.fromsquare.y+1
		if direction==east:
			x,y=self.fromsquare.x+1,self.fromsquare.y
		if direction==west:
			x,y=self.fromsquare.x-1,self.fromsquare.y
		if self.game.inbounds(x,y):
			return self.game.grid[y][x]
		return None
	def __init__(self, game,x,y):
		self.hp=1
		self.shootable=True
		self.game=game
		self.lastdirection=north
		self.x=x
		self.animationframeduration=5
		self.animationframe=0
		self.frames=0
		self.progressrate=1
		self.progress=0
		self.y=y
		self.game.grid[y][x].enemy=self
		self.fromsquare=self.game.grid[y][x]
		self.tosquare=None
		self.points=1
	def death(self):
		pass
	
	def update(self):
		self.frames+=1
		self.animationframe=self.frames//self.animationframeduration
		if self.tosquare:
			self.progress+=self.progressrate
			if self.progress>=size:
				self.progress=0
				self.fromsquare.enemy=None
				self.fromsquare=self.tosquare
				self.tosquare=None
				if self.fromsquare.haspearl:
					self.game.take_pearl(self.fromsquare)
		else:
			self.tosquare=self.nextsquare()
			if self.tosquare:
				self.tosquare.enemy=self
			
	def direction(self):
		if self.tosquare:
			result= direction(self.fromsquare.x,self.fromsquare.y,self.tosquare.x,self.tosquare.y)
			self.lastdirection=result
			return result
		else:
			return self.lastdirection
	def take_hit(self):
		self.hp-=1
		if self.hp<=0:
			self.death()
			self.game.points+=self.points
			self.game.totalpoints+=self.points
			self.game.remove_enemy(self)
			self.fromsquare.enemy=None
			if self.tosquare:
				self.tosquare.enemy=None
	def grabframe(self, list):
		return list[self.animationframe%len(list)]
	def draw(self,window):
		image=self.image()
		if image:
			offx,offy=self.offset()
			x=self.fromsquare.x*size+offx
			y=self.fromsquare.y*size+size+offy
			if self.tosquare and self.fromsquare!=self.tosquare:
				direction=self.direction()
				progress=self.progress
				if direction==north:
					y-=progress
				if direction==south:
					y+=progress
				if direction==west:
					x-=progress
				if direction==east:
					x+=progress
			window.blit(image, (int(x),int(y)))
				
	def image(self):
		return None
	def offset(self):
		return 0,0
class JellyFish(Enemy):
	blueanimation=[image('jellyfish%i.png'%(i+1)) for i in range(4)]
	orangeanimation=[image('orangejelly%i.png'%(i+1)) for i in range(4)]
	pinkanimation=[image('pinkjelly%i.png'%(i+1)) for i in range(4)]
	def __init__(self, game, x,y, color):
		Enemy.__init__(self, game, x,y)
		self.color=color
		self.game=game
		self.points=1
		self.animation=self.blueanimation
		if color=='orange':
			self.animation=self.orangeanimation
			self.progressrate=2
			self.points=3
		if color=='pink':
			self.animation=self.pinkanimation
			self.hp=3
			self.points=3
		self.pearltarget=random.choice(self.game.pearllocations)
	def image(self):
		return self.grabframe(self.animation)
	def nextsquare(self):
		direction=self.directionto(self.pearltarget[0],self.pearltarget[1])
		result= self.squarefromdirection(direction)
		if result.haswall:
			self.game.boom(self.fromsquare.x, self.fromsquare.y)
			self.game.remove_enemy(self)
			self.game.add_enemy(Explosion(self.game,self.fromsquare.x, self.fromsquare.y))
			self.fromsquare.enemy=None
			if self.tosquare:
				self.tosquare.enemy=None
		else:
			return result
			
class Explosion(Enemy):
	animation=[image('explode%i.png'%(i+1)) for i in range(3)]
	def __init__(self, game, x,y):
		Enemy.__init__(self,game,x,y)
		self.shootable=False
		self.points=0
	def update(self):
		Enemy.update(self)
		if self.animationframe==4:
			self.game.remove_enemy(self)
			self.fromsquare.enemy=None
			if self.tosquare:
				self.tosquare.enemy=None
	def image(self):
		return self.grabframe(self.animation)
	def nextsquare(self):
		return None
		
class Shark(Enemy):
	blueanimations=[ [image('shark%s%i.png'%(dir,i+1)) for i in range(4)] for dir in dirs]
	pinkanimations=[ [image('pinkshark%s%i.png'%(dir,i+1)) for i in range(4)] for dir in dirs]
	orangeanimations=[ [image('orangeshark%s%i.png'%(dir,i+1)) for i in range(4)] for dir in dirs]
	def __init__(self, game, x,y, color):
		Enemy.__init__(self, game,x,y)
		self.color=color
		if x==0:
			self.dir=east
		elif x==30:
			self.dir=west
		elif y==0:
			self.dir=south
		elif y==21:
			self.dir=north
		
		self.points=2
		self.animations=self.blueanimations
		if self.color=='orange':
			self.progressrate=3
			self.hp=3
			self.points=6
			self.animations=self.orangeanimations
		if self.color=='pink':
			self.hp=5
			self.points=3
			self.animations=self.pinkanimations
		self.pearltarget=random.choice(self.game.pearllocations)
	def image(self):
		return self.grabframe(self.animations[self.direction()])
	def nextsquare(self):
		result= self.squarefromdirection(self.dir)
		if not result:
			self.game.remove_enemy(self)
			self.fromsquare.enemy=None
			if self.tosquare:
				self.tosquare.enemy=None
			return
		if result.haswall:
			self.game.smash(result.x,result.y)
		return result
	def direction(self):
		return self.dir
	def offset(self):
		dir=self.direction()
		if dir==north:
			return 0,0
		if dir==south:
			return 0,-size
		if dir==west:
			return 0,0
		if dir==east:
			return -size,0
	
class Squid(Enemy):
	blueanimations=[ [image('squid%s%i.png'%(dir,i+1)) for i in range(4)] for dir in dirs]
	pinkanimations=[ [image('pinksquid%s%i.png'%(dir,i+1)) for i in range(4)] for dir in dirs]
	orangeanimations=[ [image('orangesquid%s%i.png'%(dir,i+1)) for i in range(4)] for dir in dirs]
	def __init__(self, game, x,y, color):
		Enemy.__init__(self, game,x,y)
		self.color=color
		self.path=None
		self.animations=self.blueanimations
		self.points=3
		if self.color=='orange':
			self.progressrate=3
			self.points=10
			self.hp=2
			self.animations=self.orangeanimations
		if self.color=='pink':
			self.hp=6
			self.points=5
			self.animations=self.pinkanimations
		self.pearltarget=random.choice(self.game.pearllocations)
	def image(self):
		return self.grabframe(self.animations[self.direction()])
	def offset(self):
		dir=self.direction()
		if dir==north:
			return 0,0
		if dir==south:
			return 0,-size
		if dir==west:
			return 0,0
		if dir==east:
			return -size,0
	def nextsquare(self):
		if self.path:
			square=self.path[0]
			if square.haswall:
				'''self.game.boom(self.fromsquare.x, self.fromsquare.y)
				self.game.remove_enemy(self)
				self.game.add_enemy(Explosion(self.game,self.fromsquare.x, self.fromsquare.y))
				self.fromsquare.enemy=None
				if self.tosquare:
					self.tosquare.enemy=None'''
				self.path=None
				return None
			else:
				self.path=self.path[1:]
				return square
		else:
			self.path=self.AStar()
			return None
	def AStar(self):
		closed_set=set()
		found_set=set()
		open_set=[]
		camefrom={}
		realscores={}
		fakescores={}
		closest=self.fromsquare
		closestdistance=self.distance(self.fromsquare)
		heappush(open_set, (self.distance(self.fromsquare), self.fromsquare))
		found_set.add(self.fromsquare)
		realscores[self.fromsquare]=0
		while open_set:
			distance, current=heappop(open_set)
			if current in closed_set:
				continue
			if distance==0:
				return self.reconstructpath(camefrom, current)
			closed_set.add(current)
			
			neighbors=self.game.neighborsquares(current)
			for neighbor in neighbors:
				if neighbor in closed_set:
					continue
				if neighbor.haswall:
					continue
				estimate=realscores[current]+1
				if neighbor not in found_set or realscores[neighbor]<estimate:
					camefrom[neighbor]=current
					realscores[neighbor]=estimate
					heappush(open_set, (self.distance(neighbor)+estimate, neighbor))
					if neighbor not in found_set:
						found_set.add(neighbor)
				if self.distance(neighbor)<closestdistance:
					closestdistance=self.distance(neighbor)
					closest=neighbor
		return self.reconstructpath(camefrom, closest)
						
	def reconstructpath(self,camefrom, current):
		if current in camefrom:
			result=self.reconstructpath(camefrom, camefrom[current])
			return result+[current]
		else:
			return [current]
				
	
	def distance(self,square):
		x,y=self.pearltarget
		return abs(square.x-x)+abs(square.y-y)