import pygame
from pygame.locals import *
from screens.game import GameScreen
from screens.tutorial import Tutorial

import random

class MenuItem:
	def __init__(self, text):
		self.text=text
		self.rect=None
		self.selected=False
		self.hovered=None

class Fish:
	def __init__(self, x,y,s,i):
		self.i=i
		self.x=x
		self.y=y
		self.s=s

class Menu:
	def __init__(self, game):
		self.menuoptions=["Resume","New Game","Tutorial","Exit"]
		self.menuoptions=[MenuItem(x) for x in self.menuoptions]
		self.game=game
		self.menufont=pygame.font.Font('../fonts/PressStart2P.ttf',16*2)
		self.enabledcolor=pygame.Color(88,248,152)
		self.selectedcolor=pygame.Color(184,248,216)
		self.disabledcolor=pygame.Color(0,64,88)
		self.backgroundcolor=pygame.Color( 0, 88, 248 )
		self.foreground=pygame.image.load('../images/titleforeground.png')
		self.background=pygame.image.load('../images/titlebackground.png')
		
		self.rightfishanimations=[ [pygame.image.load('../images/titlefish%s%i.png'%(letter,number+1)) for number in range(4)] for letter in 'abc']
		self.leftfishanimations=[ [pygame.image.load('../images/titlefish%sl%i.png'%(letter,number+1)) for number in range(4)] for letter in 'abc']
		
		self.selection_sound=pygame.mixer.Sound('../sound/menu_select.ogg')
		self.confirm_sound=pygame.mixer.Sound('../sound/menu_confirm.ogg')
		
		menuX=150
		menuY=250
		for item in self.menuoptions:
			item.rect=self.menufont.render(item.text,False, self.enabledcolor).get_rect()
			menuX=(992-item.rect.width)//2
			item.rect.topleft=(menuX,menuY)
			menuY=item.rect.bottom+20
		self.fishTop=462
		self.fishBottom=736-32
		self.leftfish=[]
		self.rightfish=[]
		self.frame=0
		self.animationframe=0
		
	def update(self):
		self.frame+=1
		#self.animationframe=self.frame//5
		#clone to loop, so we can remove
		for fish in self.leftfish[:]:
			fish.x-=fish.s
			if fish.x<0:
				self.leftfish.remove(fish)
		for fish in self.rightfish[:]:
			fish.x+=fish.s
			if fish.x>992:
				self.rightfish.remove(fish)
		
		val=random.randint(1,45)
		if val==1:
			self.rightfish.append(Fish(0,random.randint(self.fishTop,self.fishBottom),random.randint(1,3),random.randint(0,2)))
		if val==2:
			self.leftfish.append(Fish(992,random.randint(self.fishTop,self.fishBottom),random.randint(1,3),random.randint(0,2)))
	def handle_event(self, event):
		if event.type==MOUSEMOTION:
			pos=event.pos
			self.hovered=None
			for item in self.menuoptions:
				wasselected=item.selected
				item.selected=item.rect.collidepoint(pos)
				if item.selected:
					self.hovered=item
				if item.selected and not wasselected and self.isEnabled(item):
					self.selection_sound.play()
		elif event.type==MOUSEBUTTONUP:
			if event.button==1:
				self.do_selected()
	def do_selected(self):
		if self.hovered and self.isEnabled(self.hovered):
			self.confirm_sound.play()
			if self.hovered.text=='Resume':
				self.game.active_screen=self.game.paused_screen
				self.game.paused_screen=None
			elif self.hovered.text=="New Game":
				#self.game.active_screen=GameScreen(self.game,'boss')
				self.game.active_screen=GameScreen(self.game, 1)
				self.game.paused_screen=None
			elif self.hovered.text=="Tutorial":
				self.game.active_screen=Tutorial(self.game)
				self.game.paused_screen=None
			elif self.hovered.text=="Exit":
				pygame.event.post(pygame.event.Event(QUIT))
	def isEnabled(self,menu):
		if menu.text=="Resume":
			return self.game.paused_screen
		return True
	def draw(self, window):
		window.fill(self.backgroundcolor)
		window.blit(self.background,(0,0))
		
		for item in self.menuoptions:
			color=self.disabledcolor
			if self.isEnabled(item):
				color=self.enabledcolor
				if item.selected:
					color=self.selectedcolor
			else:
				continue
			blob=self.menufont.render(item.text,False, color,self.backgroundcolor)
			window.blit(blob,item.rect)
		for fish in self.leftfish:
			image=self.leftfishanimations[fish.i][(self.frame//(7-fish.s))%4]
			window.blit(image, (fish.x,fish.y))
		for fish in self.rightfish:
			image=self.rightfishanimations[fish.i][(self.frame//(7-fish.s))%4]
			window.blit(image, (fish.x,fish.y))
		window.blit(self.foreground,(0,0))
	def get_music(self):
		return "title"

